/*
 *  ReadSound - A MatLab MEX file to read an AIFF/C soundfile
 *  'after an original idea by Malcolm Slaney..'
 *  .. this one modified to use dpwe's AIF reading code etc.
 *  dpwe 1994feb10
 *
 * 1997jun06 dpwe@icsi.berkeley.edu Matlab 5 version
 * $Header: /n/crab/da/dpwe/src/SoundMex5/RCS/ReadSound.c,v 1.1 1997/06/12 00:50:09 dpwe Exp $
 */
 
#include    "dpwelib.h"		/* instead of stdio.h */
#include    <math.h>
#include    <ctype.h>
#include    "snd.h"
#include    "mex.h"

#define     MAX_TEXT    300     /* Maximum size of string option */
#define     MAX_FMT	32	/* size of format definition string */

/* #define DEBUG */

void
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    float   sampleRate;
    char    fileName[MAX_TEXT];
    char    fmtName[MAX_FMT];
    long    Length, channels;
    long    base, len;
    int     wordSize;
    int 	i;
    SOUND	*snd;
    int 	infmt;
    long   	pvl, pvb[16];

#ifdef DEBUG
    mexPrintf("ReadSound: Got %d lhs args and %d rhs args.\n", 
	      nlhs, nrhs); 
    for (i=0;i<nrhs;i++) {
	mexPrintf("Arg #%d is size %d x %d\n", 
		  (long)i, mxGetM(prhs[i]), mxGetN(prhs[i]));
    }
    for (i=0;i<nlhs;i++)
	if (plhs[i]) {
	    mexPrintf("Arg #%d is size %d x %d\n", 
		      (long)i, mxGetM(plhs[i]), mxGetN(plhs[i]));
	}
#endif /* DEBUG */

    if (nrhs < 1){
	mexPrintf("ReadSound  [D,SR,L,C] = ReadSound(FILE[,LEN[,BASE[,FMT]]])  reads a soundfile\n");
	mexPrintf("           specified by filename FILE, and returns the sounddata\n");
	mexPrintf("           scaled between -1.0 and 1.0, one row for each channel\n");
	mexPrintf("           LEN and BASE describe how much sound to return\n");
	mexPrintf("           and where to read it from (BASE defaults to 0,\n");
	mexPrintf("           LEN to -1 meaning the whole file.)\n");
	mexPrintf("           If SR is provided, it is set to the file's sampling rate.\n");
	mexPrintf("           L gets the total length and C the channel count.\n");
	mexPrintf("           FMT is a string to define the soundfile format treatment (e.g.\n");
	mexPrintf("           'AIFF', 'NeXT', 'MSWAVE', 'NIST', 'ESPS', 'PCM/Eb')\n");
	mexPrintf("           If not specified, it will attempt to guess (see sndf(3)).\n");
	return;
    }
    if (!mxIsChar(prhs[0])){
	mexErrMsgTxt("Argument must be a string (filename)");
	return;
    } else {
	/* Convert to String */
	mxGetString(prhs[0], fileName, MAX_TEXT);
    }
    if(nrhs>1)  {
	len = mxGetScalar(prhs[1]);
    } else {
	len = -1;
    }
    if(nrhs>2)  {
	base = mxGetScalar(prhs[2]);
    } else {
	base = 0;
    }
    if(nrhs > 3) {
	/* file type string is specified */
	if (!mxIsChar(prhs[3])){
	    mexErrMsgTxt("Argument 4 must be a string (filetype).");
	    return;
	}
	mxGetString(prhs[3], fmtName, MAX_FMT);
    } else {
	/* use default format name */
	fmtName[0] = '\0';
    }

#ifdef DEBUG
    mexPrintf("ReadSound: filename is '%s', len %ld, base %ld\n", 
	      fileName,len,base);
#endif /* DEBUG */
    /* reset the soundfile type determination mechanism */
    if(strlen(fmtName)==0) {
	SFSetDefaultFormat(SFF_UNKNOWN);
    } else {
	/* have a format name - try it */
	if(SFSetDefaultFormatByName(fmtName) == 0) {
	    mexErrMsgTxt("unrecognized soundfile format name");
	    return;
	}
    }
    /* Open the soundfile */
    if( (snd = sndOpen(NULL, fileName, "r")) != NULL ) {
	sampleRate = sndGetSrate(snd);
	channels   = sndGetChans(snd);
	Length     = sndGetFrames(snd);
	/* infmt	   = sndGetUFormat(snd); */
	/* invoke built-in doubles conversion */
	infmt = SFMT_DOUBLE;
	sndSetUformat(snd, infmt);
	/* we should be OK now */
	if(len <= 0)		len = Length;
	if(base+len > Length)	len = Length - base;
	if(base > Length)	{ base = 0; len = 0; }
#ifdef DEBUG 
	mexPrintf("ReadSound: file is %ld frames of %d chans, %d fmt, SR %f; read %ld+%ld\n",
		  Length, channels, infmt, sampleRate,base,len);
#endif /* DEBUG */
	if (nlhs > 0){
	    mxArray  *soundMatrix;
	    double *pd;
	     
	    soundMatrix = mxCreateDoubleMatrix(channels, len, mxREAL);
#ifdef DEBUG
	    mexPrintf("ReadSound: soundMatrix created\n");
#endif /* DEBUG */
	    plhs[0] = soundMatrix;
	    pd = mxGetPr(soundMatrix);
	    /* read in the data */
	    sndFrameSeek(snd, base, SEEK_SET);
	    sndReadFrames(snd, pd, len);
#ifdef DEBUG 
	    mexPrintf("ReadSound: frames read\n");
#endif /* DEBUG */
	}
	if (nlhs > 1){
	    mxArray *srMatrix;
	    double *pd;
	     
	    srMatrix = mxCreateDoubleMatrix(1, 1, mxREAL);
	    plhs[1] = srMatrix;
	    pd = mxGetPr(srMatrix);
	    *pd = sampleRate;
#ifdef DEBUG
	    mexPrintf("ReadSound: sampleRate created\n");
#endif /* DEBUG */
	}
	if (nlhs > 2){
	    mxArray *matrix;
	    double *pd;
	    
	    matrix = mxCreateDoubleMatrix(1, 1, mxREAL);
	    plhs[2] = matrix;
	    pd = mxGetPr(matrix);
	    *pd = Length;
	}
	if (nlhs > 3){
	    mxArray *matrix;
	    double *pd;
	     
	    matrix = mxCreateDoubleMatrix(1, 1, mxREAL);
	    plhs[3] = matrix;
	    pd = mxGetPr(matrix);
	    *pd = channels;
	}
	sndClose(snd);
    } else {
	mexErrMsgTxt("Could not open file.");
    }
    sndFree(snd);
#ifdef DEBUG
    mexPrintf("ReadSound: returning...\n");
#endif /* DEBUG */
}

